﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.CombinedIntegerVectorEncoding {
  [Item("CombinedIntegerVectorManipulator", "Base class for combined integer vector manipulators.")]
  [StorableClass]
  public abstract class CombinedIntegerVectorManipulator : SingleSuccessorOperator, ICombinedIntegerVectorManipulator, IStochasticOperator {

    #region parameters
    public ILookupParameter<IRandom> RandomParameter {
      get { return (ILookupParameter<IRandom>)Parameters["Random"]; }
    }
    public ILookupParameter<CombinedIntegerVector> ChildParameter {
      get { return (ILookupParameter<CombinedIntegerVector>)Parameters["Child"]; }
    }
    public ILookupParameter<CombinedIntegerVector> FetchedInputParameter {
      get { return (ILookupParameter<CombinedIntegerVector>)Parameters["Input"]; }
    }
    #endregion

    [StorableConstructor]
    protected CombinedIntegerVectorManipulator(bool deserializing) : base(deserializing) { }
    protected CombinedIntegerVectorManipulator(CombinedIntegerVectorManipulator original, Cloner cloner)
      : base(original, cloner) {
    }
    public CombinedIntegerVectorManipulator()
      : base() {
      Parameters.Add(new LookupParameter<IRandom>("Random"));
      Parameters.Add(new LookupParameter<CombinedIntegerVector>("Child"));
      Parameters.Add(new LookupParameter<CombinedIntegerVector>("Input"));
    }

    public sealed override IOperation Apply() {
      ChildParameter.ActualValue = Manipulate(RandomParameter.ActualValue, FetchedInputParameter.ActualValue, ChildParameter.ActualValue);
      return base.Apply();
    }

    protected abstract CombinedIntegerVector Manipulate(IRandom random, CombinedIntegerVector input, CombinedIntegerVector child);
  }
}
