﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.DecisionList {
  [Item("DecisionListManipulator", "Description missing")]
  [StorableClass]
  public abstract class DecisionListManipulator : SingleSuccessorOperator, IDecisionListManipulator {

    #region Parameter Properties
    public ILookupParameter<IRandom> RandomParameter {
      get { return (ILookupParameter<IRandom>)Parameters["Random"]; }
    }
    public ILookupParameter<DecisionList> ChildParameter {
      get { return (ILookupParameter<DecisionList>)Parameters["Child"]; }
    }
    public ILookupParameter<PercentValue> ActionMutationProbabilityParameter {
      get { return (ILookupParameter<PercentValue>)Parameters["ActionMutationProbability"]; }
    }
    #endregion

    [StorableConstructor]
    protected DecisionListManipulator(bool deserializing) : base(deserializing) { }
    protected DecisionListManipulator(DecisionListManipulator original, Cloner cloner)
      : base(original, cloner) {
    }
    public DecisionListManipulator()
      : base() {
      Parameters.Add(new LookupParameter<IRandom>("Random"));
      Parameters.Add(new LookupParameter<DecisionList>("Child"));
      Parameters.Add(new LookupParameter<PercentValue>("ActionMutationProbability"));
    }

    public override IOperation Apply() {
      Manipulate(RandomParameter.ActualValue, ChildParameter.ActualValue, ActionMutationProbabilityParameter.ActualValue.Value);
      return base.Apply();
    }

    protected abstract void Manipulate(IRandom random, DecisionList child, double actionMutationProbability);
  }
}
